package ru.dbotthepony.kstarbound.util

import kotlinx.coroutines.delay

/**
 * Allows to perform up to [maxForward] actions per given time window,
 * otherwise pauses execution
 */
class Pacer(val maxForward: Int, val delayBetween: Long) {
	private val maxForwardNanos = maxForward * delayBetween
	private var currentTime = System.nanoTime() - maxForwardNanos

	suspend fun consume(actions: Int = 1) {
		require(actions >= 1) { "Invalid amount of actions to consume: $actions" }
		val time = System.nanoTime()

		if (time - currentTime > maxForwardNanos)
			currentTime = time - maxForwardNanos

		currentTime += delayBetween * (actions - 1)
		val diff = (currentTime - time) / 1_000_000L
		currentTime += delayBetween
		if (diff > 0L) delay(diff)
	}

	companion object {
		fun actionsPerSecond(actions: Int, handicap: Int = 0): Pacer {
			return Pacer(handicap, 1_000_000_000L / actions)
		}
	}
}
